#include <gtk/gtk.h>
#include <pthread.h>
#include <sys/file.h>
#include <errno.h>
#include <libnotify/notify.h>

int finish=0;

gboolean
window_quit(GtkWidget *window,GdkEvent *event,GtkWidget *progress_window)
{
	int res;
	GtkWidget *dialog;

	if (finish == 1){
		gtk_main_quit();
		return FALSE;
	}

        dialog = gtk_message_dialog_new( NULL,
                        		 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                        		 GTK_MESSAGE_WARNING,
                        		 GTK_BUTTONS_YES_NO,
                        		 "你确定要关闭吗？\n关闭会导致数据同步出错，设备不能使用！");
	gtk_window_set_title(GTK_WINDOW(dialog),"数据同步警告");
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW(dialog),FALSE);

	GtkIconTheme *icon_theme;
        GdkPixbuf *pixbuf;
	icon_theme = gtk_icon_theme_get_default ();
	pixbuf = gtk_icon_theme_load_icon (icon_theme,
                                   	   "system-file-manager",
                                   	   16,
                                   	   0,  
                                   	   NULL);
	if (!pixbuf)
	{
		g_warning ("Couldn't load icon ");
	} else{
		gtk_window_set_icon(GTK_WINDOW(dialog),pixbuf);
		g_object_unref (pixbuf);
        }
	
	res = gtk_dialog_run (GTK_DIALOG (dialog));
	if(res == GTK_RESPONSE_YES || res == GTK_RESPONSE_APPLY)
	{
		gtk_main_quit();
		return FALSE;
	} else{
		gtk_widget_destroy(dialog);
		return TRUE;
	}
}

void 
window_minimize(GtkWidget *widget, GtkWidget *progress_window)
{
    	if (gtk_widget_get_visible (progress_window))
    	{
        	gtk_widget_hide (progress_window);
    	}
    	else
    	{
       	 	gtk_window_present (GTK_WINDOW (progress_window));
    	}
}

void
status_icon_activate_cb (GtkStatusIcon *icon,
                         GtkWidget *progress_window)
{
    	if (gtk_widget_get_visible (progress_window))
    	{
        	gtk_widget_hide (progress_window);
    	}
    	else
    	{
        	gtk_window_present (GTK_WINDOW (progress_window));
    	}
}

void 
sync_data() {
	system ("sync");
	finish = 1;
}

void 
quit_progress() {
	sleep(1);
	gtk_main_quit();
}

gboolean
update_data(GtkWidget *progress)
{
	if (finish == 1){
		pthread_t thread2;
		int ret_thrd2;

        	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), 1);
 	       	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress), "同步数据完成！");

        	notify_init ("sync data");
        	NotifyNotification * Hello = notify_notification_new ("数据同步完成", "设备已完成数据同步，可以弹出设备", "system-file-manager");
        	notify_notification_show (Hello, NULL);
        	g_object_unref(G_OBJECT(Hello));
        	notify_uninit();
	
		ret_thrd2 = pthread_create(&thread2, NULL, (void *)&quit_progress, NULL);
		if(ret_thrd2 != 0){
			printf("can't create thread\n");
			return FALSE;
		}
		
		return FALSE;
	}
	gtk_progress_bar_pulse ((GtkProgressBar *)progress);

	return TRUE;
}
 
int main(int argc, char *argv[])
{
	pthread_t thread1;
	GtkWidget *window, *fixed, *button, *image, *label1, *progress, *button2;
 	int ret_thrd1, pid_file, rc;
	
	pid_file = open("/tmp/sync_data_to_data.pid", O_CREAT | O_RDWR, 0666);
	rc = flock(pid_file, LOCK_EX | LOCK_NB);

	if(rc) {
	    	if(EWOULDBLOCK == errno)
        		return;
	}

        gtk_init(&argc, &argv);
 
        window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_window_set_title(GTK_WINDOW(window), "同步数据");
        gtk_container_set_border_width(GTK_CONTAINER(window), 10);
	gtk_window_set_resizable(GTK_WINDOW(window),FALSE);
        
        gtk_widget_set_size_request(window, 500, 200);
	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window), TRUE);
	gtk_window_set_position( GTK_WINDOW(window), GTK_WIN_POS_CENTER_ALWAYS );
 
        g_signal_connect(window, "delete_event", G_CALLBACK(window_quit), window);
 
        fixed = gtk_fixed_new();
        gtk_widget_set_size_request(fixed,500,220);

	button = gtk_button_new(); 

	image = gtk_image_new_from_icon_name ("system-file-manager",GTK_ICON_SIZE_BUTTON);


	label1 = gtk_label_new("");
	gtk_label_set_markup(label1,"正在写入缓存，请不要拔出移动存储设备，否则会丢失数据");
	

	progress = gtk_progress_bar_new();
 
        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), 0);
 
        gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress), "正在同步数据到设备");
        gtk_widget_set_size_request(progress,460,40);
 
	button2 = gtk_button_new_with_label("最小化"); 
        gtk_widget_set_size_request(button2,82,40);
        g_signal_connect(button2, "clicked", G_CALLBACK(window_minimize), window);

	gtk_fixed_put(GTK_FIXED(fixed),image,10,5);
	gtk_fixed_put(GTK_FIXED(fixed),label1,20,40);
	gtk_fixed_put(GTK_FIXED(fixed),progress,10,80);
	gtk_fixed_put(GTK_FIXED(fixed),button2,405,148);

	gtk_container_add(GTK_CONTAINER(window),fixed);

	GtkStatusIcon *status_icon;
	status_icon = gtk_status_icon_new_from_icon_name ("system-file-manager");
	gtk_status_icon_set_tooltip_text (status_icon, "同步数据");
	g_signal_connect (status_icon, "activate",
                      	  (GCallback)status_icon_activate_cb,
                      	  window);

        gtk_widget_show_all(window);
 
	ret_thrd1 = pthread_create(&thread1, NULL, (void *)&sync_data, NULL);
	if(ret_thrd1 != 0){
		printf("can't create thread\n");
		return -1;
	}

        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), 0);

	g_timeout_add(500,update_data,progress);
        gtk_main();
 
        return 0;
}
