/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 * GtkStatusbar Copyright (C) 1998 Shawn T. Amundson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
 */

#ifndef __UK_STATUSBAR_H__
#define __UK_STATUSBAR_H__

#include <gtk/gtk.h>
#include <gtk/gtkhbox.h>

#include "uk-information-panel.h"

G_BEGIN_DECLS

#define UK_TYPE_STATUSBAR            (uk_statusbar_get_type ())
#define UK_STATUSBAR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), UK_TYPE_STATUSBAR, UkStatusbar))
#define UK_STATUSBAR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), UK_TYPE_STATUSBAR, UkStatusbarClass))
#define UK_IS_STATUSBAR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), UK_TYPE_STATUSBAR))
#define UK_IS_STATUSBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), UK_TYPE_STATUSBAR))
#define UK_STATUSBAR_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), UK_TYPE_STATUSBAR, UkStatusbarClass))


typedef struct _UkStatusbar      UkStatusbar;
typedef struct _UkStatusbarClass UkStatusbarClass;

struct _UkStatusbar
{
  GtkHBox parent_widget;

  GtkWidget *GSEAL (frame);
  GtkWidget *GSEAL (label);

  GSList *GSEAL (messages);
  GSList *GSEAL (keys);

  guint GSEAL (seq_context_id);
  guint GSEAL (seq_message_id);

  GdkWindow *GSEAL (grip_window);

  guint GSEAL (has_resize_grip) : 1;

  CajaWindowInfo *window;
  UkInformationPanel *panel;
};

struct _UkStatusbarClass
{
  GtkHBoxClass parent_class;

  gpointer reserved;

  void	(*text_pushed)	(UkStatusbar	*statusbar,
			 /*guint		 context_id,*/
			 const gchar	*text);
  void	(*text_popped)	(UkStatusbar	*statusbar,
			 /*guint		 context_id,*/
			 const gchar	*text);

  /* Padding for future expansion */
  void (*_uk_reserved1) (void);
  void (*_uk_reserved2) (void);
  void (*_uk_reserved3) (void);
  void (*_uk_reserved4) (void);
};


GType      uk_statusbar_get_type     	(void) G_GNUC_CONST;
GtkWidget* uk_statusbar_new          	(void);
/* If you don't want to use contexts, 0 is a predefined global
 * context_id you can pass to push/pop/remove
 */
guint	   uk_statusbar_get_context_id	(UkStatusbar *statusbar,
					 const gchar  *context_description);
/* Returns message_id used for uk_statusbar_remove */
guint      uk_statusbar_push          	(UkStatusbar *statusbar,
					 guint	       context_id,
					 const gchar  *text);
void       uk_statusbar_pop          	(UkStatusbar *statusbar,
					 guint	       context_id);
void       uk_statusbar_remove        	(UkStatusbar *statusbar,
					 guint	       context_id,
					 guint         message_id);
void       uk_statusbar_remove_all    	(UkStatusbar *statusbar,
					 guint	       context_id);
					 

void     uk_statusbar_set_has_resize_grip (UkStatusbar *statusbar,
					    gboolean      setting);
gboolean uk_statusbar_get_has_resize_grip (UkStatusbar *statusbar);

GtkWidget* uk_statusbar_get_message_area  (UkStatusbar *statusbar);

void uk_statusbar_set_window (UkStatusbar *statusbar, CajaWindowInfo *window);

G_END_DECLS

#endif /* __UK_STATUSBAR_H__ */
